{-|
Module      : Data.Graph.Clustering.Louvain.CplusPlus
Description : C++ implementation of louvain
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : alexandre.delanoe+louvain@iscpif.fr
Stability   : experimental
Portability : POSIX

git clone https://gitlab.iscpif.fr/gargantext/clustering-louvain-cplusplus
-}

{-# LANGUAGE OverloadedStrings #-}

module Data.Graph.Clustering.Louvain.CplusPlus
  where

import Data.Graph.Clustering.Louvain.Utils (LouvainNode(..))
import Control.Monad.IO.Class (liftIO, MonadIO)
import Control.Concurrent (newEmptyMVar, takeMVar, putMVar, forkIO)
import Data.Map.Strict (Map, toList)
import Data.Maybe (maybe)
import Data.Monoid ((<>))
import Data.Text (Text)
import Data.Text (unpack)
import System.IO
import qualified Control.Foldl as Fold
import qualified Data.List as L
import qualified Data.Text    as T
-- import qualified Data.ByteString as DB
import qualified Data.Text.IO as T
import qualified Turtle as TU


cLouvain :: Text -> Map (Int, Int) Double -> IO [LouvainNode]
cLouvain _params ms = do
  let inFileD = "/tmp/louvainData.txt"
  let inFileW = "/tmp/louvainWeights.weights"
  let outBin  = "/tmp/louvainGraph.bin"
  let outTree  = "/tmp/louvainGraph.tree"
  
  let outRes  = "/tmp/louvainGraph.res"

  let louvain   = "/usr/share/louvain/louvain"
  let hierarchy = "/usr/share/louvain/hierarchy"

  writeInput inFileD ms
  putStrLn "cmdLouvain"

  let cmdLouvain   = louvain   <> " " <> inFileD <> " " <> inFileW <> " " <> outBin <> " " <> outTree
  -- let cmdLouvain   = louvain   <> " " <> inFileD <> " " <> unpack params <> " " <> inFileW <> " " <> outBin <> " " <> outTree
  putStrLn "cmdHierarchy"
  let cmdHierarchy = hierarchy <> " " <> outTree <> " -l 1 > " <> outRes
  --pure cmdLouvain
  _ <- inMVarIO $ shell (T.pack cmdLouvain)
  _ <- inMVarIO $ shell (T.pack cmdHierarchy)

  putStrLn "myResult start"
  myResult <- inMVarIO $ readOutput outRes
  putStrLn "myResult end"

  --let clean = "rm" <> " " <> L.intercalate " " [inFileD, inFileW, outBin, outTree, outRes]
  pure myResult


writeInput :: FilePath -> Map (Int, Int) Double -> IO ()
writeInput fp ns = writeFile fp (unlines ns')
  where
    ns' = map (\((n1,n2), _) -> show n1 <> " " <> show n2) (toList ns)
    --ns' = map (\((n1,n2), w) -> show n1 <> " " <> show n2 <> " " <> show w) (toList ns)

louvainNode :: [Text] -> LouvainNode
louvainNode [n1,n2] = LouvainNode (read $ T.unpack n1) (read $ T.unpack n2)
louvainNode _       = error "Louvain node not found"

readOutput :: FilePath -> IO [LouvainNode]
readOutput fp = do
  myData <- T.readFile fp
  pure $  map (\s -> louvainNode $ T.splitOn " " s) $ L.tail $ T.lines myData

shell :: TU.MonadIO io => Text -> io TU.ExitCode
shell cmd = do
  x <- TU.shell cmd TU.empty
  case x of
      TU.ExitSuccess   -> return TU.ExitSuccess
      TU.ExitFailure n -> TU.die (cmd <> " failed with exit code: " <> TU.repr n)



inMVarIO :: MonadIO m => m b -> m b
inMVarIO f = do
  mVar <- liftIO newEmptyMVar
  zVar <- f
  _ <- liftIO $ forkIO $ putMVar mVar zVar
  liftIO $ takeMVar mVar

inMVar :: b -> IO b
inMVar f = do
  mVar <- newEmptyMVar 
  let zVar = f
  _ <- liftIO $ forkIO $ putMVar mVar zVar
  liftIO $ takeMVar mVar

