{-|
Module      : Data.Louvain.Utils
Description : 
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : alexandre.delanoe+louvain@iscpif.fr
Stability   : experimental
Portability : POSIX

Tools to manage Graphs.
-}

module Data.Graph.Clustering.Louvain.Utils
  where

import Data.Graph.Inductive
import Data.List (nub)
import Data.Map.Strict (Map, toList)

data LouvainNode = LouvainNode { l_node_id :: Int
                               , l_community_id :: Int
                               } deriving (Show)

label' :: (Graph gr) => gr a b -> Edge -> Maybe b
label' gr (u,v) = lookup v (lsuc gr u)

shortest_path :: (Real b, Graph gr) => gr a b -> Node -> Node -> Maybe Path
shortest_path graph node_1 node_2 = sp node_1 node_2 graph

mkGraph' :: [LEdge b] -> Gr () b
mkGraph' es = mkGraph ns es
    where
        ns :: [LNode ()]
        ns = zip [1.. (fromIntegral . length) ns'] (repeat ())
            where ns' = nub $ concat (Prelude.map edge2nodes es)

edge2nodes :: LEdge b -> [Node]
edge2nodes (a,b,_) = [a,b]

map2graph :: Map (Node, Node) b -> Gr () b
map2graph m = mkGraph' $ map (\((n1,n2), w) -> (n1,n2,w)) $ toList m


